import {ab2str, str2ab} from './arrayutils.js';

export function exportedKeyToPem(key, type) {
    let exportedAsString = ab2str(key);
    let exportedAsBase64 = window.btoa(exportedAsString);
    return `-----BEGIN ${type.toUpperCase()} KEY-----\n${exportedAsBase64}\n-----END ${type.toUpperCase()} KEY-----`;
}

export function pemToKey(pemKey, type) {
    const base64 = pemKey.replace(`-----BEGIN ${type.toUpperCase()} KEY-----`, '').replace(`-----END ${type.toUpperCase()} KEY-----`, '').trim();
    const binaryDerString = window.atob(base64);
    const binaryDer = str2ab(binaryDerString);
    return window.crypto.subtle.importKey(
        "pkcs8",
        binaryDer,
        {
            name: "Ed25519",
        },
        true,
        ["sign"],
    );
}

export async function genKey() {
  // Generate keys
    const { publicKey, privateKey } = await crypto.subtle.generateKey(
      {
        name: "Ed25519",
      },
      true,
      ["sign", "verify"],
    );
    const exportedPubkey = await crypto.subtle.exportKey("spki", publicKey);
    const exportedPrivkey = await crypto.subtle.exportKey("pkcs8", privateKey);
    // const privkey = await crypto.subtle.wrapKey("pkcs8", privateKey, wrapkey, { name: "AES-KW" });
    const key = {
        privateKey: exportedPrivkey,
        publicKey: exportedPubkey
    }
    return key;
}
