import { concatUint8Arrays } from './arrayutils.js';
import { keccak } from './keccak.js';

class keccakKDF {
    constructor(encryptionKeyLength = 16, privateKeyLength = 32) {
        this.encryptionKeyLength = encryptionKeyLength;
        this.privateKeyLength = privateKeyLength;
    }
    kdf(salt) {
        const input = concatUint8Arrays(this.currentKey, salt);
        const outputLength = this.encryptionKeyLength + this.privateKeyLength;
        const output = keccak.SHAKE256(input, outputLength);
        const encryptionKey = output.slice(0, this.encryptionKeyLength);
        const newPrivateKey = output.slice(this.encryptionKeyLength, outputLength);
        this.currentKey = newPrivateKey;
        return encryptionKey;
    }
    init(initialKey, salt) {
        this.currentKey = initialKey;
        return this.kdf(salt);
    }
    next(salt=(new Uint8Array(0))) {
        if (!this.currentKey) throw new Error("KDF not initialized");
        return this.kdf(salt);
    }

}
export { keccakKDF };
